/*
 * *************************************************
 * Copyright (c) 2018 QMTH. All Rights Reserved.
 * Created by Deason on 2018-11-02 10:01:13.
 * *************************************************
 */

package cn.com.qmth.examcloud.core.print.common.utils;

import org.apache.commons.io.IOUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import java.io.*;
import java.nio.charset.Charset;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.UUID;
import java.util.zip.ZipEntry;
import java.util.zip.ZipOutputStream;

/**
 * @author: fengdesheng
 * @since: 2018/11/2
 */
public class FileUtils {
    private static Logger log = LoggerFactory.getLogger(FileUtils.class);

    public static String randomUUID() {
        return UUID.randomUUID().toString().replace("-", "");
    }

    /**
     * 生成日期目录
     * PS: yyyyMM
     */
    public static String dateDir() {
        final String pattern = "yyyyMM";
        return "/" + new SimpleDateFormat(pattern).format(new Date()) + "/";
    }

    /**
     * 生成新的文件名(带后缀)
     * PS: uuid.xyz
     */
    public static String newFileName(String fileName) {
        String uuid = UUID.randomUUID().toString().replaceAll("-", "");
        if (fileName != null && !"".equals(fileName)) {
            int index = fileName.lastIndexOf(".");
            if (index > 0 && index < fileName.length() - 1) {
                //获取原来文件的后缀名
                return uuid + fileName.substring(index);
            }
        }
        return uuid;
    }

    /**
     * 获取文件名(包含后缀名)
     */
    public static String getFileName(String filePath) {
        if (filePath == null || "".equals(filePath)) {
            return "";
        }
        filePath = filePath.replace("\\", "/");
        int index = filePath.lastIndexOf("/");
        if (index > -1) {
            return filePath.substring(index + 1);
        }
        return filePath;
    }

    /**
     * 创建文件目录
     */
    public static boolean makeDirs(String path) {
        if (path == null || "".equals(path)) {
            return false;
        }
        File folder = new File(path);
        if (!folder.exists()) {
            return folder.mkdirs();
        }
        return true;
    }

    /**
     * 文件压缩
     *
     * @param target  待压缩的目录或文件
     * @param zipFile 压缩后的文件
     */
    public static boolean zip(File target, File zipFile) {
        if (target == null || !target.exists()) {
            log.error("目录或文件不能为空！");
            return false;
        }

        if (zipFile == null) {
            log.error("待压缩的文件不能为空！");
            return false;
        }

        ZipOutputStream zipStream = null;
        OutputStream outStream = null;
        try {
            if (!zipFile.exists()) {
                zipFile.createNewFile();
            }

            outStream = new FileOutputStream(zipFile);
            zipStream = new ZipOutputStream(outStream, Charset.forName("UTF-8"));

            if (target.isDirectory()) {
                File[] files = target.listFiles();
                if (files.length == 0) {
                    log.error("文件夹内未找到任何文件！");
                    return false;
                }

                for (File file : files) {
                    doZip(zipStream, file, null);
                }
            } else {
                doZip(zipStream, target, null);
            }
            return true;
        } catch (IOException e) {
            log.error(e.getMessage(), e);
            return false;
        } finally {
            try {
                if (zipStream != null) {
                    zipStream.close();
                }
                if (outStream != null) {
                    outStream.close();
                }
            } catch (IOException e) {
                //ignore
            }
        }
    }

    private static void doZip(ZipOutputStream zipStream, File curFile, String dirName) throws IOException {
        if (dirName == null) dirName = "";
        String entryName = dirName + curFile.getName();
        //log.debug("entryName:" + entryName);

        if (curFile.isDirectory()) {
            String subDirName = entryName + File.separator;
            File[] files = curFile.listFiles();
            if (files.length > 0) {
                for (File file : files) {
                    doZip(zipStream, file, subDirName);
                }
            } else {
                //空文件夹暂不处理
                //zipStream.putNextEntry(new ZipEntry(subDirName));
                //zipStream.closeEntry();
            }
        } else {
            int len;
            byte[] bytes = new byte[1024];
            InputStream is = new FileInputStream(curFile);
            zipStream.putNextEntry(new ZipEntry(entryName));
            while ((len = is.read(bytes)) > 0) {
                zipStream.write(bytes, 0, len);
            }
            IOUtils.closeQuietly(is);
            zipStream.closeEntry();
        }
    }

}